Package: ggplotplus
Title: Universal Design-Oriented Enhancements for 'ggplot2'
Version: 0.5.5
Authors@R: 
    person("Alex", "Bajcz", email = "bajcz003@umn.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5909-4676"))
BugReports: https://github.com/MAISRC/ggplotplus/issues
Description: A collection of enhancements to 'ggplot2', with a focus on creating Universally Designed, accessible graphs easily and quickly.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/MAISRC/ggplotplus
RoxygenNote: 7.3.3
Language: en-US
Imports: ggplot2 (>= 4.0.1), rlang, S7, grid, gtable, scales,
        viridisLite, polyclip, ggrepel (>= 0.9.0), dplyr, stats
Suggests: testthat (>= 3.0.0), ragg (>= 1.5.2), patchwork, cowplot
Config/testthat/edition: 3
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-09 19:53:30 UTC; Bajcz
Author: Alex Bajcz [aut, cre] (ORCID: <https://orcid.org/0000-0002-5909-4676>)
Maintainer: Alex Bajcz <bajcz003@umn.edu>
Repository: CRAN
Date/Publication: 2026-06-17 13:40:08 UTC
