# Generated by roxygen2: do not edit by hand

export(add_shape_plus)
export(direct_labels_plus)
export(geom_jitter_plus)
export(geom_point_plus)
export(geom_point_plus_shapes)
export(ggplotplus_to_cowplot)
export(ggplotplus_to_patchwork)
export(gridlines_plus)
export(scale_continuous_plus)
export(theme_plus)
export(yaxis_title_plus)
import(ggplot2)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,ungroup)
importFrom(ggplot2,class_ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,update_ggplot)
importFrom(polyclip,polyclip)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rlang,list2)
importFrom(scales,pal_viridis)
importFrom(scales,pretty_breaks)
importFrom(stats,median)
importFrom(utils,globalVariables)
importFrom(viridisLite,viridis)
